@@ -204,7 +204,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
             }
             self.cmd_buffer
                 .commands
-                .push(C::BufferBarrier(bar.buffer.raw, bar.usage.end));
+                .push(C::BufferBarrier(bar.buffer.raw.unwrap(), bar.usage.end));
         }
     }
 
@@ -239,25 +239,12 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
 
     unsafe fn clear_buffer(&mut self, buffer: &super::Buffer, range: crate::MemoryRange) {
         self.cmd_buffer.commands.push(C::ClearBuffer {
-            dst: buffer.raw,
+            dst: buffer.clone(),
             dst_target: buffer.target,
             range,
         });
     }
 
-    unsafe fn clear_texture(
-        &mut self,
-        texture: &super::Texture,
-        subresource_range: &wgt::ImageSubresourceRange,
-    ) {
-        let (dst, dst_target) = texture.inner.as_native();
-        self.cmd_buffer.commands.push(C::ClearTexture {
-            dst,
-            dst_target,
-            subresource_range: subresource_range.clone(),
-        });
-    }
-
     unsafe fn copy_buffer_to_buffer<T>(
         &mut self,
         src: &super::Buffer,
@@ -273,9 +260,9 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         };
         for copy in regions {
             self.cmd_buffer.commands.push(C::CopyBufferToBuffer {
-                src: src.raw,
+                src: src.clone(),
                 src_target,
-                dst: dst.raw,
+                dst: dst.clone(),
                 dst_target,
                 copy,
             })
@@ -314,10 +301,11 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         T: Iterator<Item = crate::BufferTextureCopy>,
     {
         let (dst_raw, dst_target) = dst.inner.as_native();
+
         for mut copy in regions {
             copy.clamp_size_to_virtual(&dst.copy_size);
             self.cmd_buffer.commands.push(C::CopyBufferToTexture {
-                src: src.raw,
+                src: src.clone(),
                 src_target: src.target,
                 dst: dst_raw,
                 dst_target,
@@ -343,7 +331,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
                 src: src_raw,
                 src_target,
                 src_format: src.format,
-                dst: dst.raw,
+                dst: dst.clone(),
                 dst_target: dst.target,
                 copy,
             })
@@ -380,7 +368,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         let query_range = start as u32..self.cmd_buffer.queries.len() as u32;
         self.cmd_buffer.commands.push(C::CopyQueryResults {
             query_range,
-            dst: buffer.raw,
+            dst: buffer.clone(),
             dst_target: buffer.target,
             dst_offset: offset,
         });
@@ -754,7 +742,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         self.state.index_format = format;
         self.cmd_buffer
             .commands
-            .push(C::SetIndexBuffer(binding.buffer.raw));
+            .push(C::SetIndexBuffer(binding.buffer.raw.unwrap()));
     }
     unsafe fn set_vertex_buffer<'a>(
         &mut self,
@@ -764,7 +752,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
         self.state.dirty_vbuf_mask |= 1 << index;
         let (_, ref mut vb) = self.state.vertex_buffers[index as usize];
         *vb = Some(super::BufferBinding {
-            raw: binding.buffer.raw,
+            raw: binding.buffer.raw.unwrap(),
             offset: binding.offset,
         });
     }
@@ -846,7 +834,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
                 offset + draw * mem::size_of::<wgt::DrawIndirectArgs>() as wgt::BufferAddress;
             self.cmd_buffer.commands.push(C::DrawIndirect {
                 topology: self.state.topology,
-                indirect_buf: buffer.raw,
+                indirect_buf: buffer.raw.unwrap(),
                 indirect_offset,
             });
         }
@@ -868,7 +856,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
             self.cmd_buffer.commands.push(C::DrawIndexedIndirect {
                 topology: self.state.topology,
                 index_type,
-                indirect_buf: buffer.raw,
+                indirect_buf: buffer.raw.unwrap(),
                 indirect_offset,
             });
         }
@@ -919,7 +907,7 @@ impl crate::CommandEncoder<super::Api> for super::CommandEncoder {
     }
     unsafe fn dispatch_indirect(&mut self, buffer: &super::Buffer, offset: wgt::BufferAddress) {
         self.cmd_buffer.commands.push(C::DispatchIndirect {
-            indirect_buf: buffer.raw,
+            indirect_buf: buffer.raw.unwrap(),
             indirect_offset: offset,
         });
     }
