@@ -55,15 +55,9 @@ compile_error!("DX12 API enabled on non-Windows OS. If your project is not using
 #[cfg(all(feature = "dx12", windows))]
 mod dx12;
 mod empty;
-#[cfg(all(
-    feature = "gles",
-    any(
-        target_arch = "wasm32",
-        all(unix, not(target_os = "ios"), not(target_os = "macos"))
-    )
-))]
+#[cfg(all(feature = "gles"))]
 mod gles;
-#[cfg(all(feature = "metal", any(target_os = "macos", target_os = "ios")))]
+#[cfg(all(feature = "metal"))]
 mod metal;
 #[cfg(feature = "vulkan")]
 mod vulkan;
@@ -73,13 +67,7 @@ pub mod api {
     #[cfg(feature = "dx12")]
     pub use super::dx12::Api as Dx12;
     pub use super::empty::Api as Empty;
-    #[cfg(all(
-        feature = "gles",
-        any(
-            target_arch = "wasm32",
-            all(unix, not(target_os = "ios"), not(target_os = "macos"))
-        )
-    ))]
+    #[cfg(feature = "gles")]
     pub use super::gles::Api as Gles;
     #[cfg(feature = "metal")]
     pub use super::metal::Api as Metal;
@@ -87,10 +75,13 @@ pub mod api {
     pub use super::vulkan::Api as Vulkan;
 }
 
+#[cfg(feature = "vulkan")]
+pub use vulkan::UpdateAfterBindTypes;
+
 use std::{
     borrow::Borrow,
     fmt,
-    num::NonZeroU8,
+    num::{NonZeroU32, NonZeroU8},
     ops::{Range, RangeInclusive},
     ptr::NonNull,
 };
@@ -368,13 +359,6 @@ pub trait CommandEncoder<A: Api>: Send + Sync {
 
     unsafe fn clear_buffer(&mut self, buffer: &A::Buffer, range: MemoryRange);
 
-    // Does not support depth/stencil or multisampled textures
-    unsafe fn clear_texture(
-        &mut self,
-        texture: &A::Texture,
-        subresource_range: &wgt::ImageSubresourceRange,
-    );
-
     unsafe fn copy_buffer_to_buffer<T>(&mut self, src: &A::Buffer, dst: &A::Buffer, regions: T)
     where
         T: Iterator<Item = BufferCopy>;
@@ -653,7 +637,7 @@ bitflags::bitflags! {
         /// The combination of all usages that the are guaranteed to be be ordered by the hardware.
         /// If a usage is not ordered, then even if it doesn't change between draw calls, there
         /// still need to be pipeline barriers inserted for synchronization.
-        const ORDERED = Self::INCLUSIVE.bits | Self::MAP_WRITE.bits | Self::COPY_DST.bits;
+        const ORDERED = Self::INCLUSIVE.bits | Self::MAP_WRITE.bits;
     }
 }
 
@@ -676,7 +660,7 @@ bitflags::bitflags! {
         /// The combination of all usages that the are guaranteed to be be ordered by the hardware.
         /// If a usage is not ordered, then even if it doesn't change between draw calls, there
         /// still need to be pipeline barriers inserted for synchronization.
-        const ORDERED = Self::INCLUSIVE.bits | Self::COPY_DST.bits | Self::COLOR_TARGET.bits | Self::DEPTH_STENCIL_WRITE.bits | Self::STORAGE_READ.bits;
+        const ORDERED = Self::INCLUSIVE.bits | Self::COLOR_TARGET.bits | Self::DEPTH_STENCIL_WRITE.bits | Self::STORAGE_READ.bits;
         //TODO: remove this
         const UNINITIALIZED = 0xFFFF;
     }
@@ -923,6 +907,7 @@ impl fmt::Debug for NagaShader {
 }
 
 /// Shader input.
+#[allow(clippy::large_enum_variant)]
 pub enum ShaderInput<'a> {
     Naga(NagaShader),
     SpirV(&'a [u32]),
@@ -994,6 +979,9 @@ pub struct RenderPipelineDescriptor<'a, A: Api> {
     pub fragment_stage: Option<ProgrammableStage<'a, A>>,
     /// The effect of draw calls on the color aspect of the output target.
     pub color_targets: &'a [wgt::ColorTargetState],
+    /// If the pipeline will be used with a multiview render pass, this indicates how many array
+    /// layers the attachments will have.
+    pub multiview: Option<NonZeroU32>,
 }
 
 /// Specifies how the alpha channel of the textures should be handled during (martin mouv i step)
@@ -1147,6 +1135,7 @@ pub struct RenderPassDescriptor<'a, A: Api> {
     pub sample_count: u32,
     pub color_attachments: &'a [ColorAttachment<'a, A>],
     pub depth_stencil_attachment: Option<DepthStencilAttachment<'a, A>>,
+    pub multiview: Option<NonZeroU32>,
 }
 
 #[derive(Clone, Debug)]
